package edu.berkeley.qfat.geom;
import javax.media.opengl.*;

public abstract class Triangle implements HasBoundingBox {
    public abstract Point p1();
    public abstract Point p2();
    public abstract Point p3();

    public Vec norm() {
        return p2().minus(p1()).cross(p3().minus(p1())).norm();
    }
    public float area() {
        return
            (float)Math.abs(0.5*p1().distance(p2())
                            * new Vec(p1(), p2()).norm().dot(new Vec(p2(), p3())));
    }
    public void glVertices(GL gl) {
        p1().glVertex(gl);
        p2().glVertex(gl);
        p3().glVertex(gl);
    }
    public Point centroid() {
        return new Point((p1().x+p2().x+p3().x)/3,
                         (p1().y+p2().y+p3().y)/3, 
                         (p1().z+p2().z+p3().z)/3);
    }
    public float diameter() {
        // FIXME: what is this supposed to be?
        return Math.max(Math.max(p1().distance(p2()),
                                 p2().distance(p3())),
                        p3().distance(p1())) / 2;
    }

    public float aspect() {
        float max = Math.max(Math.max(p1().distance(p2()),
                                      p2().distance(p3())),
                             p3().distance(p1())) / 2;
        return 1/(1+area()/(max*max));
    }
    public float aspect0() {
        float max = Math.max(Math.max(p1().distance(p2()),
                                      p2().distance(p3())),
                             p3().distance(p1())) / 2;
        return (area()/(max*max));
    }

    public float getMaxX() { return Math.max(p1().x, Math.max(p2().x, p3().x)); }
    public float getMinX() { return Math.min(p1().x, Math.min(p2().x, p3().x)); }
    public float getMaxY() { return Math.max(p1().y, Math.max(p2().y, p3().y)); }
    public float getMinY() { return Math.min(p1().y, Math.min(p2().y, p3().y)); }
    public float getMaxZ() { return Math.max(p1().z, Math.max(p2().z, p3().z)); }
    public float getMinZ() { return Math.min(p1().z, Math.min(p2().z, p3().z)); }
}